import {world, system, ItemStack, EnchantmentTypes} from '@minecraft/server'
import { Enchantments } from "Enchantments.js";

const forbiddentItems = ["dbe:", "shulker"]

system.beforeEvents.watchdogTerminate.subscribe((eventData)=>{
    eventData.cancel = true
})

system.runTimeout(()=>{
    system.runInterval(()=>{
        let allPlayers = world.getAllPlayers()

        //Item Changing Event
        for(let player of allPlayers){
            let currentHeldItemStatusProperty = player.getDynamicProperty(`playerHeldItemStatus`)
            let currentPropertyJson
            let inventory = player.getComponent("inventory").container
            let selectedSlot = player.selectedSlot
            let itemHolding = inventory.getItem(selectedSlot)
            let propertyInterface = {item:undefined, slot:undefined, id:undefined}
            let changed = false

            if(itemHolding){
                propertyInterface.item = itemHolding.typeId
                if(itemHolding.getLore().length>0){
                    propertyInterface.id = itemHolding.getLore()[0]
                }
            }
            propertyInterface.slot = selectedSlot
            //world.sendMessage(`${currentHeldItemStatusProperty} -- ${JSON.stringify(propertyInterface)}`)
            if(currentHeldItemStatusProperty != undefined){
                if(JSON.stringify(propertyInterface) != currentHeldItemStatusProperty){
                    currentPropertyJson = JSON.parse(currentHeldItemStatusProperty)
                    if(currentPropertyJson.slot != selectedSlot) {
                        changed = true
                    }
                    if(itemHolding){
                        if(currentPropertyJson.item != itemHolding.typeId){
                            changed = true
                        }                    
                    }else{
                        if(currentPropertyJson.item != undefined){
                            changed = true
                        }
                        if(currentPropertyJson.item == undefined && itemHolding == undefined){
                            changed = false
                        }
                    }
                    if(changed){
                        //player.sendMessage(`Item Changed - ${JSON.stringify(propertyInterface)}`)
                        onChanged(player, inventory, propertyInterface, currentPropertyJson)
                        player.setDynamicProperty(`playerHeldItemStatus`,JSON.stringify(propertyInterface))
                    }else{
                        player.setDynamicProperty(`playerHeldItemStatus`,JSON.stringify(propertyInterface))
                    }
                }
            }else{
                player.setDynamicProperty(`playerHeldItemStatus`,JSON.stringify(propertyInterface))
            }
            
            //Teleport Backpack Entity to Player
            if(itemHolding != undefined){
                if(itemHolding.typeId.includes("dbe:backpack")){
                    if(propertyInterface.id != undefined){
                        let bpEntityQuery = {tags:[propertyInterface.id]}
                        let bpEntities = player.dimension.getEntities(bpEntityQuery)
                        for(let bpEntity of bpEntities){
                            let newLocation = player.location
                            newLocation.y = newLocation.y+1.5
                            bpEntity.teleport(newLocation)
                        }
                    }
                }
            }                      
        }

        //Record Items When Backpack Close
        let bpCloseQuery = {tags:["close"]}
        let bpCloseEntities = getBackpackEntity(bpCloseQuery)
        for(let bpEntity of bpCloseEntities){           
            let bpEntityInventory = bpEntity.getComponent("inventory").container
            let playerQuery = {type:"minecraft:player", closest:1, location:bpEntity.location}
            let playerEntity = bpEntity.dimension.getEntities(playerQuery)
            for(let player of playerEntity){
                let inventory = player.getComponent("inventory").container
                let item = inventory.getItem(player.selectedSlot)
                if(!item) continue               
                if(item.getLore()[0] != bpEntity.getTags()[0]) continue
                inventory.setItem(player.selectedSlot, item)
            }
            bpEntity.removeTag("close")
        }
    },1)

    //Manage Inactive Backpacks
    system.runInterval(()=>{
        let bpEntities = getBackpackEntity({families:["dbe"]})
        for(let bpEntity of bpEntities){
            let playerQuery = {type:"minecraft:player", location:bpEntity.location, maxDistance:3}
            let players = bpEntity.dimension.getEntities(playerQuery)
            let newLocation = bpEntity.location
            newLocation.y = -64
            if(players.length==0){
                bpEntity.teleport(newLocation)
            }
        }
    },60)
},60)

function onChanged(player, playerInventory, propertyDataCurrent, propertyDataOld){
    let currentItem = playerInventory.getItem(propertyDataCurrent.slot) 

    if(currentItem != undefined){
        if(currentItem.typeId.includes("dbe:backpack")){
            let lore = currentItem.getLore()            
            if(lore.length == 0){ //Create New Backpack ID
                let bpId = `dbe_id:${Math.floor(Math.random() * 9999)}`
                currentItem.setLore([bpId])
                playerInventory.setItem(propertyDataCurrent.slot, currentItem)
                let bpEntity = player.dimension.spawnEntity("dbe:container_entity_temp", player.location)
                bpEntity.triggerEvent(backpackType(currentItem))
                bpEntity.addTag(bpId)
                bpEntity.nameTag = backpackName(currentItem)
            }else{ //Open Old Backpack
                let bpEntityQuery = {tags:[`${propertyDataOld.id}`]}
                let bpEntities = player.dimension.getEntities(bpEntityQuery)
                for(let bpEntity of bpEntities){
                    closeBackpack(bpEntity, player, propertyDataOld)
                }
                openBackpack(player, currentItem)               
            }
        }else{                     
            let bpEntityQuery = {tags:[`${propertyDataOld.id}`]}
            let bpEntities = player.dimension.getEntities(bpEntityQuery)
            for(let bpEntity of bpEntities){
                let bpEntityInv = bpEntity.getComponent("inventory").container
                for(let slot=0; slot<bpEntityInv.size; slot++){
                    let item = bpEntityInv.getItem(slot)
                    if(!item) continue
                    if(forbiddentItems.find((itemName) => item.typeId.includes(itemName))){
                        bpEntity.dimension.spawnItem(item, bpEntity.location)
                        bpEntityInv.setItem(slot, undefined)
                    }
                }
                closeBackpack(bpEntity, player, propertyDataOld)
            }
        }
    }else{
        if(propertyDataOld.id != undefined){
            let bpEntityQuery = {tags:[`${propertyDataOld.id}`]}
            let bpEntities = player.dimension.getEntities(bpEntityQuery)
            for(let bpEntity of bpEntities){
                let bpEntityInv = bpEntity.getComponent("inventory").container
                for(let slot=0; slot<bpEntityInv.size; slot++){
                    let item = bpEntityInv.getItem(slot)
                    if(!item) continue
                    if(forbiddentItems.find((itemName) => item.typeId.includes(itemName))){
                        bpEntity.dimension.spawnItem(item, bpEntity.location)
                        bpEntityInv.setItem(slot, undefined)
                    }
                }
                closeBackpack(bpEntity, player, propertyDataOld)      
            }
        }
    }   
}

world.afterEvents.chatSend.subscribe((eventData)=>{
    const {message, sender} = eventData
    let cmd = message.split(" ")
    let isPlayerOP = sender.isOp() ? true : sender.hasTag("op") ? true : false

    if(cmd.length>1){
        if(cmd[0].trim() == "!dbe"){
            switch(cmd[1]){
                case "clear":
                    if(cmd[2] == "bp"){
                        if(!isPlayerOP){
                            sender.sendMessage(`[Backpack+] OP required to use this command.`)
                            return
                        }
                        var bpQuery = {families:["backpack"]}
                        var bpEntities = getBackpackEntity(bpQuery)
                        let count = 0
                        for(let bpEntity of bpEntities){
                            world.setDynamicProperty(bpEntity.getTags()[0], undefined)
                            bpEntity.kill()
                            count++                
                        }
                        sender.sendMessage(`§c[Backpack+] Cleared all ${count} backpacks.`)
                    }                    
                break
        
                case "reset":
                    var inv = sender.getComponent("inventory").container
                    var item = inv.getItem(sender.selectedSlot)
                    if(!item){
                        sender.sendMessage(`§c[Backpack+] Hold a backpack item`)
                        return
                    }
                    if(!item.typeId.includes("backpack")){
                        sender.sendMessage(`§c[Backpack+] Hold a backpack item`)
                        return
                    }
                    inv.setItem(sender.selectedSlot, new ItemStack(item.typeId))
                break

                case "see":
                    if(!isPlayerOP){
                        sender.sendMessage(`[Backpack+] OP required to use this command.`)
                        return
                    }
                    var id = cmd[2]
                    var bpQuery = {tags:[`dbe_id:${id}`]}
                    var bpEntities = getBackpackEntity(bpQuery)
                    if(bpEntities.length==0){
                        sender.sendMessage(`§c[Backpack+] No backpack with that ID found!`)
                        return
                    }
                    for(let bpEntity of bpEntities){
                        let bpInv = bpEntity.getComponent("inventory").container
                        let itemList = "§eBackpack Items: "
                        for(let slot=0; slot<bpInv.size; slot++){
                            let item = bpInv.getItem(slot)
                            if(!item) continue
                            let itemName = ""
                            let itemNameArray = item.typeId.split(":")[1].split("_")
                            for(let itemStr of itemNameArray){
                                itemName +=  itemStr.charAt(0).toUpperCase() + itemStr.slice(1) + " ";
                            }
                            itemList += `§e${itemName.trim()} x${item.amount}§a, `
                        }
                        sender.sendMessage(itemList)
                    }
                break

                case "retrieve":
                    if(!isPlayerOP){
                        sender.sendMessage(`[Backpack+] OP required to use this command.`)
                        return
                    }
                    var id = cmd[2]
                    var bpQuery = {tags:[`dbe_id:${id}`]}
                    var bpEntities = getBackpackEntity(bpQuery)
                    if(bpEntities.length==0){
                        sender.sendMessage(`§c[Backpack+] No backpack with that ID found!`)
                        return
                    }
                    for(let bpEntity of bpEntities){
                        let bpInv = bpEntity.getComponent("inventory").container
                        for(let slot=0; slot<bpInv.size; slot++){
                            let item = bpInv.getItem(slot)
                            if(!item) continue
                            sender.dimension.spawnItem(item,sender.location)
                            bpInv.setItem(slot, undefined)
                        }                       
                    }                    
                    world.setDynamicProperty(`dbe_id:${id}`, undefined)
                    sender.sendMessage(`§e[Backpack+] Successfully retrieve items.`)
                break

                case "set":
                    if(!isPlayerOP){
                        sender.sendMessage(`[Backpack+] OP required to use this command.`)
                        return
                    }
                    var id = cmd[2]
                    var inv = sender.getComponent("inventory").container
                    var item = inv.getItem(sender.selectedSlot)
                    if(!item){
                        sender.sendMessage(`§c[Backpack+] Hold a backpack item`)
                        return
                    }
                    if(!item.typeId.includes("backpack")){
                        sender.sendMessage(`§c[Backpack+] Hold a backpack item`)
                        return
                    }
                    item.setLore([`dbe_id:${id}`])
                    inv.setItem(sender.selectedSlot, item)
                break

                case "delete":
                    if(!isPlayerOP){
                        sender.sendMessage(`[Backpack+] OP required to use this command.`)
                        return
                    }
                    var id = cmd[2]
                    var bpQuery = {tags:[`dbe_id:${id}`]}
                    var bpEntities = getBackpackEntity(bpQuery)
                    if(bpEntities.length==0){
                        sender.sendMessage(`§c[Backpack+] No backpack with that ID found!`)
                        return
                    }
                    for(let bpEntity of bpEntities){
                        let bpInv = bpEntity.getComponent("inventory").container
                        for(let slot=0; slot<bpInv.size; slot++){
                            if(item){
                                bpInv.setItem(slot, undefined)
                            }                           
                        }
                        bpEntity.kill()                    
                    }                 
                    world.setDynamicProperty(`dbe_id:${id}`, undefined)
                    sender.sendMessage(`§e[Backpack+] Backpack Deleted.`)
                break

                case "debug":
                    let DynamicPropertyByteCount = world.getDynamicPropertyTotalByteCount()
                    sender.sendMessage(`Byte Count:` + DynamicPropertyByteCount)
                    let debugProperty = ""
                    for(let x=0; x<32767; x++){
                        debugProperty = debugProperty + "x"
                    }                    
                    world.setDynamicProperty("debug", debugProperty)
                    DynamicPropertyByteCount = world.getDynamicPropertyTotalByteCount()
                    sender.sendMessage(`After Debug Byte Count:` + DynamicPropertyByteCount)
                    world.setDynamicProperty("debug", undefined)
                break

                case "list":
                    let backpackEntities = sender.dimension.getEntities({families:["dbe"]})
                    let idList = "§e[Backpack+] ID List: "
                    for(let backpackEntity of backpackEntities){
                        idList = idList + `${backpackEntity.getTags()[0].split(":")[1]}, ` 
                    }
                    sender.sendMessage(idList)
                break
            }
        }
    }else{
        if(cmd[0].trim() == "!dbe"){
            sender.sendMessage(`§e<Backpack+ Commands>`)       
            sender.sendMessage(`§e!dbe reset - Reset the backpack item your holding.`)
            if(isPlayerOP){
                sender.sendMessage(`§e!dbe clear bp - Deletes all backpacks and item recovery.`)
                sender.sendMessage(`§e!dbe list - See all backpack IDs.`)
                sender.sendMessage(`§e!dbe see <ID:int> - See specific backpack items.`)               
                sender.sendMessage(`§e!dbe retrieve <ID:int> - Retrieve backpack items.`)
                sender.sendMessage(`§e!dbe set <ID:int> - Manually set your backpack id.`)
                sender.sendMessage(`§e!dbe delete <ID:int> - Delete backpack and recovery items.`)
                //sender.sendMessage(`§e!dbe delete inactive - Delete inactive backpack and recovery items. Inactive backpacks are the those that don't have items.`)
            }
        }
    }  
})

function backpackType(item){
    let typeId = item.typeId
    switch(typeId){
        case "dbe:backpack":
            return "small"
        case "dbe:backpack_medium":
            return "medium"
        case "dbe:backpack_big":
            return "big"
        case "dbe:backpack_xl":
            return "xl"
    }
}

function backpackName(item){
    let typeId = item.typeId
    switch(typeId){
        case "dbe:backpack":
            return "Small Backpack"
        case "dbe:backpack_medium":
            return "Medium Backpack"
        case "dbe:backpack_big":
            return "Big Backpack"
        case "dbe:backpack_xl":
            return "XL Backpack"
    }
}

function getBackpackEntity(query){
    let dimensions = ["overworld"]
    let entityList = []
    for(let dimension of dimensions){
        let bpEntities = world.getDimension(dimension).getEntities(query)
        if(bpEntities.length>0){
            for(let bpEntity of bpEntities){
                entityList.push(bpEntity)
            }           
        }
    }
    return entityList
}

function openBackpack(player,bpItem){
    let bpOldEntityQuery = {tags:[bpItem.getLore()[0]]}
    let bpOldEntities = getBackpackEntity(bpOldEntityQuery)
    if(bpOldEntities.length == 0){
        player.sendMessage(`§c[Backpack+] Unable to retrieve backpack items.`)
        player.sendMessage(`§c[Backpack+] Will attempt to recover items.`)
        bpOldEntities = recoverItems(player, bpItem.getLore()[0])
        if(!bpOldEntities) return
    }
    let bpEntity = player.dimension.spawnEntity("dbe:container_entity_temp", player.location)
    bpEntity.triggerEvent(backpackType(bpItem))
    let bpInventory = bpEntity.getComponent("inventory").container      
    for(let bpOldEntity of bpOldEntities){
        let bpOldEntityInventory = bpOldEntity.getComponent("inventory").container
        for(let index=0; index<bpOldEntityInventory.size; index++){
            let item = bpOldEntityInventory.getItem(index)
            if(!item) continue
            bpInventory.setItem(index, item)
            bpOldEntityInventory.setItem(index, undefined)
        }
        bpOldEntity.kill()
        try{
            bpOldEntity.triggerEvent("despawn2")
        }catch(e){}
    }
    bpEntity.addTag(bpItem.getLore()[0])
    bpEntity.nameTag = backpackName(bpItem)   
}

function closeBackpack(backpackEntity, player, propertyDataOld){
    let newLocation = backpackEntity.location
    newLocation.y = -64
    backpackEntity.teleport(newLocation)
    /*
    let bpOldEntityQuery = {tags:[`${propertyDataOld.id}`]}
    let bpOldEntities = player.dimension.getEntities(bpOldEntityQuery)   
    let newLore = []
    for(let bpOldEntity of bpOldEntities){                     
        let bpOldInventory = bpOldEntity.getComponent("inventory").container
        newLore = recordItems(propertyDataOld.id, bpOldInventory)
    }
    let playerHeldItem = player.getComponent("inventory").container.getItem(propertyDataOld.slot)
    playerHeldItem.setLore(newLore)
    player.getComponent("inventory").container.setItem(propertyDataOld.slot, playerHeldItem)*/
}

function recordItems(id, inventory, bpEntity){    
    let count = 0;
    let countOverflow = 0
    let backupItemList = []
    let lore = [`${id}`]
    for(let slot=0; slot<inventory.size; slot++){        
        let item = inventory.getItem(slot)
        if(!item) continue
        if(forbiddentItems.find((itemName) => item.typeId.includes(itemName))){
            bpEntity.dimension.spawnItem(item, bpEntity.location)
            inventory.setItem(slot, undefined)
            continue
        }
        let itemName = ""
        let itemNameArray = item.typeId.split(":")[1].split("_")
        for(let itemStr of itemNameArray){
            itemName +=  itemStr.charAt(0).toUpperCase() + itemStr.slice(1) + " ";
        }
        if(count < 5){
            lore.push(`§7${itemName}x${item.amount}`)     
        }else{
            countOverflow++
        }       
        count++        
    }
    if(countOverflow>0){
        lore.push(`§7and ${countOverflow} more...`)
    }
    world.setDynamicProperty(`${id}`, JSON.stringify(backupItemList))
    //world.sendMessage(`${world.getDynamicPropertyTotalByteCount()}`)
    //world.sendMessage(world.getDynamicProperty(`${id}`))
    return lore
}

function getItemProperties(item){
    let itemInterface = {id:undefined, name:undefined, amount:undefined, lore:undefined, durability:undefined, enchant:undefined}
    let enchantArray = []
    itemInterface.id = item.typeId
    itemInterface.amount = item.amount
    if(item.hasComponent("durability")){
        itemInterface.durability = item.getComponent("durability").damage
    }   
    if(item.nameTag) itemInterface.name = item.nameTag
    if(item.getLore().length > 0) itemInterface.lore = item.getLore()
    if(enchantList.length>0){
        for(let enchant of enchantList){
            let enchantInterface = {enchantName:undefined,level:undefined}
            enchantInterface.enchantName = enchant.type.id
            enchantInterface.level = enchant.level
            enchantArray.push(enchantInterface)
        }
    itemInterface.enchant = enchantArray
    }
    return itemInterface
}

function recoverItems(player, id){
    let itemProperties = world.getDynamicProperty(id)
    if(itemProperties != undefined){
        let playerInv = player.getComponent("inventory").container
        let bpEntity = player.dimension.spawnEntity("dbe:container_entity_temp", player.location)
        bpEntity.addTag(id)
        bpEntity.nameTag = backpackName(playerInv.getItem(player.selectedSlot))
        bpEntity.triggerEvent(backpackType(playerInv.getItem(player.selectedSlot)))
        let bpInv = bpEntity.getComponent("inventory").container
        let itemPropertiesParse = JSON.parse(itemProperties)
        for(let itemProperty of itemPropertiesParse){
            let item = new ItemStack(itemProperty.id, itemProperty.amount)
            if(itemProperty.name) item.nameTag = itemProperty.name
            if(itemProperty.lore) item.setLore(itemProperty.lore)
            if(itemProperty.durability) item.getComponent("durability").damage = itemProperty.durability
            if(itemProperty.enchant){
                for(let enchant of itemProperty.enchant){                   
                    item = Enchantments.addEnchant(item, {type:EnchantmentTypes.get(enchant.enchantName), level:enchant.level})
                }
            }
            bpInv.addItem(item)
        }
        player.sendMessage(`§e[Backpack+] Items successfully recovered.`)
        return [bpEntity]
    }else{
        player.sendMessage(`§c[Backpack+] Unable to find recovery backup.`)
        return false
    }
}