import { world, system } from "@minecraft/server";
import { getScore, formatNumber } from "../utils";

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    let powerScore = getScore(player, "dbe:power");
    powerScore = Math.min(powerScore, 100);
    const increasePower =
      powerScore > 0
        ? `\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n[increase_power_${Math.round(
            (powerScore * 30) / 100
          )}]`
        : ` `;
    player.onScreenDisplay.setTitle(`${increasePower}`);
  }
}, 0.25);

const generateActionBar = (
  player,
  value,
  maxValue,
  color,
  barLength,
  isInCreative
) => {
  const percentage = Math.min((value / maxValue) * 100, 100);
  const barCount = Math.floor((percentage / 100) * barLength);
  let displayValue;

  if (isInCreative) {
    displayValue = "∞";
  } else {
    displayValue = value > 0 ? formatNumber(Math.min(value, maxValue)) : "0";
  }

  const coloredBars = `§l${Array(barCount)
    .fill(`§${color}|`)
    .join("")}§f${Array(barLength - barCount)
    .fill("|")
    .join("")}`;

  return `${coloredBars} §r§${color}${displayValue}`;
};

system.runInterval(() => {
  world.getPlayers().forEach((player) => {
    const energy = getScore(player, "dbe:energy");
    const maxEnergy = getScore(player, "dbe:energy3");
    const health = getScore(player, "dbe:health");
    const maxHealth = getScore(player, "dbe:health2");

    const levelScore = getScore(player, "dbe:level");
    const form2Score = getScore(player, "dbe:form2");
    const energy5Score = getScore(player, "dbe:energy5");

    const oozaruTag = player.hasTag("dbe:oozaru");
    const formHudTag = player.hasTag("dbe:select_form_hud");
    const levelTag = player.hasTag("dbe:level_hud");
    const xpTag = player.hasTag("dbe:level_xp_hud");
    const tpTag = player.hasTag("dbe:tp_hud");
    const hasKaiokenTag = player.hasTag("dbe:has_kaioken");
    const inSurvival = player.hasTag("dbe:in_survival");
    const inCreative = player.hasTag("dbe:in_creative");

    player.runCommandAsync(`tag @s[m=!1] add dbe:in_survival`);
    player.runCommandAsync(`tag @s[m=1] add dbe:in_creative`);
    player.runCommandAsync(`tag @s[m=1] remove dbe:in_survival`);
    player.runCommandAsync(`tag @s[m=!1] remove dbe:in_creative`);

    const raceScore = getScore(player, "dbe:race");
    const hasLegendaryTest =
      player.hasTag("dbe:has_legendary") && raceScore === 1;
    const isMidnightTest = player.hasTag("dbe:is_midnight") && raceScore === 1;

    const actionBarText = [];
    const shouldAddEmptyLine =
      hasKaiokenTag ||
      hasLegendaryTest ||
      isMidnightTest ||
      !(hasKaiokenTag || hasLegendaryTest || isMidnightTest) ||
      formTag;

    if (shouldAddEmptyLine) {
      actionBarText.push("\n\n");
    }

    if (levelTag) {
      actionBarText.push(
        { text: "§6" },
        { translate: "level" },
        { text: ` ${formatNumber(getScore(player, "dbe:level"))}\n` }
      );
    } else if (xpTag) {
      actionBarText.push(
        { text: "§6" },
        { translate: "XP" },
        {
          text: ` ${formatNumber(
            getScore(player, "dbe:level2")
          )}/${formatNumber(getScore(player, "dbe:level3"))}\n`,
        }
      );
    } else if (tpTag) {
      actionBarText.push(
        { text: "§6" },
        { translate: "TP" },
        {
          text: ` ${formatNumber(getScore(player, "dbe:tp"))}\n`,
        }
      );
    }

    if (inSurvival) {
      actionBarText.push(
        generateActionBar(player, energy, maxEnergy, "b", 30, false),
        ` \n${generateActionBar(player, health, maxHealth, "c", 15, false)}`
      );
    } else if (inCreative) {
      actionBarText.push(
        generateActionBar(player, maxEnergy, maxEnergy, "b", 30, true),
        ` \n${generateActionBar(player, maxHealth, maxHealth, "c", 15, true)}`
      );
    }

    if (shouldAddEmptyLine) {
      actionBarText.push(
        `\n\n${hasKaiokenTag ? "\ue1f2 " : ""}${
          hasLegendaryTest ? "\ue1f1 " : ""
        }${isMidnightTest ? "\ue1f0 " : ""}`
      );
    }

    if (formHudTag) {
      const formConditions = {
        1: [
          {
            level: 0,
            form2: 0,
            translate: "without_form",
            requires_midnight: false,
            requires_level: false,
          },
          {
            level: 0,
            form2: 1,
            translate: "oozaru",
            requires_midnight: true,
            requires_level: false,
          },
          {
            level: 8,
            form2: 2,
            translate: "false_super_saiyan",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 20,
            form2: 3,
            translate: "super_saiyan",
            requires_midnight: false,
            requires_level: true,

          },
          {
            level: 30,
            form2: 4,
            translate: "super_saiyan_2",
            requires_midnight: false,
            requires_level: true,

          },
          {
            level: 45,
            form2: 5,
            translate: "super_saiyan_3",
            requires_midnight: false,
            requires_level: true,

          },
          {
            level: 55,
            form2: 6,
            translate: "golden_oozaru",
            requires_midnight: false,
            requires_level: true,

          },
          {
            level: 70,
            form2: 7,
            translate: "super_saiyan_4",
            requires_midnight: false,
            requires_level: true,

          },
          {
            level: 75,
            form2: 8,
            translate: "super_saiyan_god",
            requires_midnight: false,
            requires_level: true,

          },
          {
            level: 80,
            form2: 9,
            translate: "super_saiyan_blue",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 80,
            form2: 9,
            translate: "super_saiyan_rose",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 90,
            form2: 10,
            translate: "super_saiyan_blue_shinka",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 90,
            form2: 10,
            translate: "super_saiyan_rose_shinka",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 120,
            form2: 11,
            translate: "super_saiyan_blue_3",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 120,
            form2: 11,
            translate: "super_saiyan_rose_3",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 125,
            form2: 12,
            translate: "super_saiyan_4_limit_breaker",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 130,
            form2: 13,
            translate: "super_saiyan_5",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 140,
            form2: 14,
            translate: "beast",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 150,
            form2: 15,
            translate: "ultra_ego",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 160,
            form2: 16,
            translate: "super_saiyan_4_evolved",
            requires_midnight: false,
            requires_level: true,
          },
      
        ],
        2: [
          {
            level: 0,
            form2: 0,
            translate: "without_form",
            requires_midnight: false,
            requires_level: false,
          },
          {
            level: 5,
            form2: 1,
            translate: "namekian_giant",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 10,
            form2: 2,
            translate: "namekian_full_released",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 33,
            form2: 3,
            translate: "namekian_god",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 50,
            form2: 4,
            translate: "namekian_red_eyed",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 80,
            form2: 5,
            translate: "golden_namekian",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 120,
            form2: 6,
            translate: "namekian_unleashed",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 140,
            form2: 7,
            translate: "metal_namekian",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 160,
            form2: 8,
            translate: "orange_namekian",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 200,
            form2: 9,
            translate: "enraged_orange_namekian",
            requires_midnight: false,
            requires_level: true,
          },
        ],
        3: [
          {
            level: 0,
            form2: 0,
            translate: "without_form",
            requires_midnight: false,
            requires_level: false,
          },
          {
            level: 6,
            form2: 1,
            translate: "arcosian_second_form",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 8,
            form2: 2,
            translate: "arcosian_third_form",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 20,
            form2: 3,
            translate: "arcosian_fourth_form",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 60,
            form2: 4,
            translate: "arcosian_full_power",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 75,
            form2: 5,
            translate: "arcosian_fifth_form",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 90,
            form2: 6,
            translate: "golden_arcosian",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 110,
            form2: 7,
            translate: "golden_arcosian_fifth_form",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 130,
            form2: 8,
            translate: "black_arcosian",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 150,
            form2: 9,
            translate: "black_arcosian_fifth_form",
            requires_midnight: false,
            requires_level: true,
          },
        ],
        4: [
          {
            level: 0,
            form2: 0,
            translate: "without_form",
            requires_midnight: false,
            requires_level: false,
          },
          {
            level: 5,
            form2: 1,
            translate: "bio_semi",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 10,
            form2: 2,
            translate: "bio_perfect",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 33,
            form2: 3,
            translate: "bio_fullpower",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 100,
            form2: 4,
            translate: "bio_goldenform",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 140,
            form2: 5,
            translate: "imperfectmax_form",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 160,
            form2: 6,
            translate: "semimax_form",
            requires_midnight: false,
            requires_level: true,
          },
          {
            level: 180,
            form2: 7,
            translate: "perfectmax_form",
            requires_midnight: false,
            requires_level: true,
          },
        ],
      6: [
        {
          level: 0,
          form2: 0,
          translate: "without_form",
          requires_midnight: false,
          requires_level: false,
        },
        {
          level: 5,
          form2: 1,
          translate: "demon_false",
          requires_midnight: false,
          requires_level: true,
        },
        {
          level: 10,
          form2: 2,
          translate: "demon_evil",
          requires_midnight: false,
          requires_level: true,
        },
        {
          level: 33,
          form2: 3,
          translate: "demon_rage",
          requires_midnight: false,
          requires_level: true,
        },
        {
          level: 50,
          form2: 4,
          translate: "demon_shadow",
          requires_midnight: false,
          requires_level: true,
        },
        {
          level: 70,
          form2: 5,
          translate: "demon_dark",
          requires_midnight: false,
          requires_level: true,
        },
        {
          level: 85,
          form2: 6,
          translate: "demon_god",
          requires_midnight: false,
          requires_level: true,
        },
        {
          level: 95,
          form2: 7,
          translate: "demon_king",
          requires_midnight: false,
          requires_level: true,
        },
      ],
      7: [
        {
          level: 0,
          form2: 0,
          translate: "without_form",
          requires_midnight: false,
          requires_level: false,
        },
        {
          level: 12,
          form2: 1,
          translate: "majin_ultra",
          requires_midnight: false,
          requires_level: true,
        },
        {
          level: 25,
          form2: 2,
          translate: "majin_benign",
          requires_midnight: false,
          requires_level: true,
        },
        {
          level: 35,
          form2: 3,
          translate: "majin_evil",
          requires_midnight: false,
          requires_level: true,
        },
        {
          level: 50,
          form2: 4,
          translate: "majin_super",
          requires_midnight: false,
          requires_level: true,
        },
        {
          level: 75,
          form2: 5,
          translate: "majin_ultimate",
          requires_midnight: false,
          requires_level: true,
        },
        {
          level: 87,
          form2: 6,
          translate: "majin_pure",
          requires_midnight: false,
          requires_level: true,
        },
        {
          level: 99,
          form2: 7,
          translate: "majin_dark",
          requires_midnight: false,
          requires_level: true,
        },
      ],
    };
      const formCondition = formConditions[raceScore].find(
        (condition) => condition.form2 === form2Score
      );

      if (formCondition) {
        const actionBarTextData = [
          { text: "\n\n §f" },
          { translate: formCondition.translate || "Unknown Form" },
          { text: "\n\n" },
        ];

        // Verifique se não está em modo criativo antes de adicionar as condições
        if (!inCreative) {
          if (formCondition.requires_midnight) {
            actionBarTextData.push({ translate: "requires_midnight" });
          }
          if (formCondition.requires_level) {
            actionBarTextData.push({ translate: "requires_level" });
            actionBarTextData.push({
              text: levelScore >= formCondition.level ? "§a" : "§c",
            });
            actionBarTextData.push({ text: ` ${formCondition.level || 0}` });
          }
        }

        actionBarTextData.push({ text: "\n\n" });

        actionBarText.push(...actionBarTextData);
      }
    }

    player.runCommandAsync(
      "scoreboard players operation @s dbe:objective = @s dbe:mission4"
    );
    player.runCommandAsync(
      "scoreboard players operation @s dbe:mission5 = @s dbe:mission"
    );

    if (
      player.hasTag("dbe:on_mission") &&
      getScore(player, "dbe:mission") == 1
    ) {
      actionBarText.push({
        rawtext: [
          {
            text: "\n\n§f",
          },
          {
            translate: "mission.info",
          },
          {
            text: " \ue1fb\n\n§f",
          },
          {
            translate: "objective",
          },
          {
            text: ": §c",
          },
          {
            translate: "mission.objective_1",
          },
          {
            text: `\n\n§f- ${getScore(player, "dbe:objective")}/${getScore(
              player,
              "dbe:objective2"
            )}`,
          },
        ],
      });
    } else if (
      player.hasTag("dbe:on_mission") &&
      getScore(player, "dbe:mission") == 24
    ) {
      actionBarText.push({
        rawtext: [
          {
            text: "\n\n§f",
          },
          {
            translate: "mission.info",
          },
          {
            text: " \ue1fb\n\n§f",
          },
          {
            translate: "objective",
          },
          {
            text: ": §c",
          },
          {
            translate: "mission.objective_24",
          },
          {
            text: `\n\n§f- ${getScore(player, "dbe:objective")}/${getScore(
              player,
              "dbe:objective2"
            )}`,
          },
        ],
      });
    } else if (
      player.hasTag("dbe:on_mission") &&
      getScore(player, "dbe:mission") == 25
    ) {
      actionBarText.push({
        rawtext: [
          {
            text: "\n\n§f",
          },
          {
            translate: "mission.info",
          },
          {
            text: " \ue1fb\n\n§f",
          },
          {
            translate: "objective",
          },
          {
            text: ": §c",
          },
          {
            translate: "mission.objective_25",
          },
          {
            text: `\n\n§f- ${getScore(player, "dbe:objective")}/${getScore(
              player,
              "dbe:objective2"
            )}`,
          },
        ],
      });
    } else if (
      player.hasTag("dbe:on_mission") &&
      getScore(player, "dbe:mission") == 51
    ) {
      actionBarText.push({
        rawtext: [
          {
            text: "\n\n§f",
          },
          {
            translate: "mission.info",
          },
          {
            text: " \ue1fb\n\n§f",
          },
          {
            translate: "objective",
          },
          {
            text: ": §c",
          },
          {
            translate: "mission.objective_51",
          },
          {
            text: `\n\n§f- ${getScore(player, "dbe:objective")}/${getScore(
              player,
              "dbe:objective2"
            )}`,
          },
        ],
      });
    } else if (
      player.hasTag("dbe:on_mission") &&
      getScore(player, "dbe:mission") == 54
    ) {
      actionBarText.push({
        rawtext: [
          {
            text: "\n\n§f",
          },
          {
            translate: "mission.info",
          },
          {
            text: " \ue1fb\n\n§f",
          },
          {
            translate: "objective",
          },
          {
            text: ": §c",
          },
          {
            translate: "mission.objective_54",
          },
          {
            text: `\n\n§f- ${getScore(player, "dbe:objective")}/${getScore(
              player,
              "dbe:objective2"
            )}`,
          },
        ],
      });
    } else if (player.hasTag("dbe:on_mission")) {
      actionBarText.push({
        rawtext: [
          {
            text: "\n\n§f",
          },
          {
            translate: "mission.info",
          },
          {
            text: " \ue1fb\n\n§f",
          },
          {
            translate: "objective",
          },
          {
            text: ": §c",
          },
          {
            translate: `mission.objective_${getScore(player, "dbe:mission")}`,
          },
        ],
      });
    }

    player.onScreenDisplay.setActionBar(actionBarText);
  });
}, 0.25);

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    if (!player.hasTag("dbe:scouter")) continue;

    const filter = {
      location: player.location,
      minDistance: 1,
      maxDistance: 32,
      excludeFamilies: ["inanimate"],
    };

    try {
      for (const entity of world
        .getDimension(player.dimension.id)
        .getEntities(filter)) {
        if (entity.typeId.startsWith("dbe:") || entity.typeId === "player") {
          var h = entity.getComponent("minecraft:health");
          if (h == undefined) continue;
          const healthComponent = h.currentValue;
          const healthValue = getScore(entity, "dbe:health2");
          const healthString = `§c${Math.round(healthComponent)}`;
          if (getScore(entity, "dbe:health2") > 0) {
            const healthShow = ` / ${Math.round(healthValue)}`;
            player.onScreenDisplay.updateSubtitle({
              rawtext: [
                {
                  translate: `entity.${entity.typeId}.name`,
                },
                {
                  text: ` ${healthString}`,
                },
                {
                  text: `${healthShow}`,
                },
              ],
            });
          }
        }
      }
    } catch (error) {
      // Lidar com erros, se necessário
    }
  }
}, 0.25);
