import { world, system } from "@minecraft/server";
import { timeoutPeriod, addonName, addonDescription,commands } from "./constant";


export function getPlayerByName(playerName) {
    return Array.from(world.getPlayers()).find(
        (player) => player.name === playerName
    );
}

export function teleport(migrator, acceptor) {
    system.run(() => {
        world.getDimension(getPlayerByName(migrator).dimension.id).runCommandAsync(`tp ${migrator} ${acceptor}`);
    });
}


export function tpa(args, player, teleportRequests) {

    if (args.length === 0) {
        player.sendMessage("§6Usage: !tpa §c<Playername>");
        return;
    }
    if (args[0] === "help") {
        player.sendMessage(`${addonName}\n${addonDescription}`);
        player.sendMessage("-----------------------------------");
        player.sendMessage(commands.join("\n"));
        return;
    }

    const targetPlayerName = args[0];
    const targetPlayer = getPlayerByName(targetPlayerName);

    if (player.name === targetPlayerName) {
        player.sendMessage("§6§lYou can't teleport to yourself!");
        return;
    }

    if (targetPlayer) {
        const request = {
            requestor: player.name,
            acceptor: targetPlayerName,
            finalLocation: targetPlayerName,
            countdown: timeoutPeriod,
        };
        teleportRequests.push(request);

        player.sendMessage(`§6Teleport request sent to §c${targetPlayerName}.`);
        player.sendMessage("§6To cancel this request, type §c!tpacancel");

        targetPlayer.sendMessage(`§6§c${player.name} §6has requested to teleport to you.`);
        targetPlayer.sendMessage('§6To teleport, type §c!tpaccept.');
        targetPlayer.sendMessage('§6To deny this request, type §c!tpdeny.');
        targetPlayer.sendMessage('§6This request will timeout after §c120 seconds.');

        const intervalId = system.runInterval(() => {
            request.countdown -= 1;
            if (!teleportRequests.includes(request)) {
                system.clearRun(intervalId);
                return;
            }
            if (request.countdown <= 0) {
                system.clearRun(intervalId);

                const index = teleportRequests.indexOf(request);
                if (index !== -1) {
                    teleportRequests.splice(index, 1);
                }

                player.sendMessage(`§6Teleport request to §c${targetPlayerName} §6has timed out.`);
                targetPlayer.sendMessage(`§6The Teleport request from §c${player.name} §6has timed out.`);
            }
        }, 20);
    }
    else
    {
        player.sendMessage(`§6Player §c${targetPlayerName} §6not found.`);
    }
}



export function tpahereall(args, player, teleportRequests) {
    if (args.length !== 0) {
        player.sendMessage("§6Usage: §c!tpahereall");
        return;
    }

    const targetPlayers = Array.from(world.getPlayers());
    if (targetPlayers.length === 1) {
        player.sendMessage("§6§lThere are no other players online.");
        return;
    }

    targetPlayers.forEach((targetPlayer) => {
        if (targetPlayer.name !== player.name) {
            const request = {
                requestor: player.name,
                acceptor: targetPlayer.name,
                finalLocation: player.name,
                countdown: timeoutPeriod,
            };

            teleportRequests.push(request);

            const intervalId = system.runInterval(() => {
                request.countdown -= 1;

                if (!teleportRequests.includes(request)) {
                    system.clearRun(intervalId);
                    return;
                }

                if (request.countdown <= 0) {
                    system.clearRun(intervalId);

                    const index = teleportRequests.indexOf(request);
                    if (index !== -1) {
                        teleportRequests.splice(index, 1);
                    }

                    player.sendMessage(`§6Teleport request to §c${targetPlayer.name} §6has timed out.`);
                    targetPlayer.sendMessage(`§6The Teleport request from §c${player.name} §6has timed out.`);
                }
            }, 20);

            targetPlayer.sendMessage(`§6§c${player.name} §6has requested that you teleport to them.`);
            targetPlayer.sendMessage('§6To teleport, type §c!tpaccept.');
            targetPlayer.sendMessage('§6To deny this request, type §c!tpdeny.');
            targetPlayer.sendMessage('§6This request will timeout after §c120 seconds.');
        }
    });

    player.sendMessage(`§6Teleport requests sent to all §c${targetPlayers.length - 1} players.`);
}



export function tpahere(args, player, teleportRequests) {
    if (args.length === 0) {
        player.sendMessage("§6Usage: §c!tpahere <Playername>");
        return;
    }

    const targetPlayerName = args[0];

    if (player.name === targetPlayerName) {
        player.sendMessage("§6§lYou can't teleport to yourself!");
        return;
    }

    const targetPlayer = getPlayerByName(targetPlayerName);

    if (targetPlayer) {
        const request = {
            requestor: player.name,
            acceptor: targetPlayerName,
            finalLocation: player.name,
            countdown: timeoutPeriod,
        };

        teleportRequests.push(request);

        const intervalId = system.runInterval(() => {
            request.countdown -= 1;

            if (!teleportRequests.includes(request)) {
                system.clearRun(intervalId);
                return;
            }

            if (request.countdown <= 0) {
                system.clearRun(intervalId);

                const index = teleportRequests.indexOf(request);
                if (index !== -1) {
                    teleportRequests.splice(index, 1);
                }

                player.sendMessage(`§6Teleport here request to §c${targetPlayerName} §6has timed out.`);
                targetPlayer.sendMessage(`§6The Teleport here request from §c${player.name} §6has timed out.`);
            }
        }, 20);

        player.sendMessage(`§6Teleport here request sent to §c${targetPlayerName}.`);
        player.sendMessage("§6To cancel this request, type §c!tpacancel");

        targetPlayer.sendMessage(`§6§c${player.name} §6has requested that you teleport to them.`);
        targetPlayer.sendMessage('§6To teleport, type §c!tpaccept.');
        targetPlayer.sendMessage('§6To deny this request, type §c!tpdeny.');
        targetPlayer.sendMessage('§6This request will timeout after §c120 seconds.');
    }
    else {
        player.sendMessage(`§6Player §c${targetPlayerName} §6not found.`);
    }
}



export function tpacancel(args, player, teleportRequests) {
    const canceledRequests = teleportRequests.filter(
        (req) => req.requestor === player.name
    );

    if (canceledRequests.length !== 0) {
        canceledRequests.forEach((request) => {
            const index = teleportRequests.indexOf(request);
            teleportRequests.splice(index, 1);
            player.sendMessage(`§6Teleport request to §c${request.acceptor} §6has been canceled.`);
            const targetPlayer = getPlayerByName(request.acceptor);
            if (targetPlayer) {
                targetPlayer.sendMessage(`§6§c${player.name} §6has canceled the teleport request.`);
            }
        });
    }
    else {
        player.sendMessage("§cYou don't have any pending teleport requests.");
    }
}


export function tpaccept(args, player, teleportRequests) {
    if (args.length !== 0) {
        player.sendMessage("§6Invalid Command. Usage: §c!tpaccept");
        return;
    }

    const index = teleportRequests.findIndex(
        (req) => req.acceptor === player.name
    );
    if (index !== -1) {
        const request = teleportRequests[index];
        player.sendMessage("§6Teleport request accepted.");
        const requester = getPlayerByName(request.requestor);
        if (requester) {
            requester.sendMessage(`§6§c${player.name} §6has accepted your teleport request.`);
            const migrator = (request.finalLocation !== request.requestor) ? request.requestor : request.acceptor;
            teleport(migrator, request.finalLocation);
            teleportRequests.splice(index, 1);
        }
    }
    else {
        player.sendMessage("§cYou don't have any pending teleport requests.");
    }
}


export function tpdeny(args, player, teleportRequests) {
    if (args.length !== 0) {
        player.sendMessage("§6Invalid Command. Usage: §c!tpdeny");
        return;
    }

    const index = teleportRequests.findIndex(
        (req) => req.acceptor === player.name
    );
    if (index !== -1) {
        const request = teleportRequests[index];
        const requester = getPlayerByName(request.requestor);
        if (requester) {
            requester.sendMessage(`§6§c${player.name} §6has denied your teleport request.`);
        }
        player.sendMessage(`§6Teleport request for §c${request.requestor} §6has been denied!`);
        teleportRequests.splice(index, 1);
    }
    else {
        player.sendMessage("§cYou don't have any pending teleport requests.");
    }
}