import { world, system } from "@minecraft/server";
import { ActionFormData } from "@minecraft/server-ui";
import { getScore, formatNumber } from "../utils";
import * as kaiokenUI from "./kaioken";
import * as missionsUI from "./missions";



world.beforeEvents.itemUse.subscribe((data) => {
  const player = data.source;
  switch (data.itemStack.typeId) {
    case "dbe:menu":
      if (getScore(player, "dbe:energy5") != 0) {
        if (player.hasTag("dbe:kaioken")) {
          kaiokenUI.kaioken(player);
        } else if (player.hasTag("dbe:kaioken_x3")) {
          kaiokenUI.kaioken_x3(player);
        } else if (player.hasTag("dbe:kaioken_x10")) {
          kaiokenUI.kaioken_x10(player);
        } else if (player.hasTag("dbe:kaioken_x20")) {
          kaiokenUI.kaioken_x20(player);
        } else if (
          !(
            player.hasTag("dbe:kaioken") &&
            player.hasTag("dbe:kaioken_x3") &&
            player.hasTag("dbe:kaioken_x10") &&
            player.hasTag("dbe:kaioken_x20")
          )
        ) {
          system.runTimeout(() => {
            menu(player);
          }, 1);
        }
        player.runCommandAsync("playsound random.click @s ~~~ 0.1 3");
      } else if (getScore(player, "dbe:energy5") == 0) {
        player.runCommandAsync("playsound note.pling @s ~~~ 0.1");
        player.runCommandAsync("execute as @s at @s run function start");
        player.sendMessage({ rawtext: [{ translate: "warn.error.1" }] });
      }
  }
});

//@menu
export function menu(player) {
  const form = new ActionFormData();
  form.title("ui.menu.title");
  form.button("ui.menu.button_2");
  form.button({ rawtext: [{ translate: "kaioken" }, { text: " (OFF)" }] });
  form.button("entity.dbe:shadow_dummy.name");
  form.button("ui.menu.button_5");
  form.button("ui.menu.button_6");
  form.show(player).then((response) => {
    switch (response.selection) {
      case 0:
        system.runTimeout(() => {
          attributes(player);
        }, 1);
        break;
      case 1:
        if (
          getScore(player, "dbe:mission") == 93 &&
          !player.hasTag("dbe:safe_zone_gamemode")
        ) {
          missionsUI.endSaga(player);
        } else if (!player.hasTag("dbe:safe_zone_gamemode")) {
          missionsUI[`mission_${getScore(player, "dbe:mission")}`](player);
        } else
          system.runTimeout(() => {
            player.sendMessage({ rawtext: [{ translate: "warn.error.3" }] });
          }, 1);
        break;
      case 2:
        if (player.hasTag("dbe:kaioken_off")) {
          kaiokenUI.kaioken(player);
          player.addTag("dbe:has_kaioken");
          player.triggerEvent("dbe:kaioken_true");
          player.runCommandAsync("particle dbe:kaioken_transformation ~~~");
          player.triggerEvent("dbe:kaioken");
        } else
          player.sendMessage({
            rawtext: [
              {
                translate: "mission.error.1",
                with: {
                  rawtext: [
                    { score: { name: "@s", objective: "dbe:mission3" } },
                  ],
                },
              },
            ],
          });
        break;
      case 3:
        if (
          !player.hasTag("dbe:shadow_dummy") &&
          !player.hasTag("dbe:safe_zone_gamemode")
        ) {
          player.triggerEvent("dbe:shadow_dummy");
        } else player.sendMessage({ rawtext: [{ translate: "warn.error.2" }] });
        break;
        case 4:
          system.runTimeout(() => {
            teleport(player);
          }, 1);
          break;
          case 5:
            player.teleport({ x: -59, y: 234, z: 68 }, { dimension: world.getDimension("overworld") });
            break;
      }
    });
  }

//attributes
export function attributes(player) {
  const form = new ActionFormData();
  form.title("ui.attributes.title");
  form.body(
    `TP: §7${formatNumber(
      getScore(player, "dbe:tp")
    )}\n\n§rCON §7${formatNumber(
      getScore(player, "dbe:statscon")
    )}\n§rSTR §7${formatNumber(
      getScore(player, "dbe:statsstr")
    )}\n§rSPE §7${formatNumber(
      getScore(player, "dbe:statsspe")
    )}\n§rUC ${getScore(player, "dbe:uc")}x: §7${getScore(
      player,
      "dbe:uc2"
    )} TP`
  );
  form.button("");
  form.button("+");
  form.button("+");
  form.button("+");
  form.button("+");
  form.show(player).then((response) => {
    const ucPrice = getScore(player, "dbe:uc2");
    switch (response.selection) {
      case 0:
        if (player.hasTag("dbe:kaioken")) {
          kaiokenSelection.kaioken(player);
        } else if (player.hasTag("dbe:kaioken_x3")) {
          kaiokenSelection.kaioken_x3(player);
        } else if (player.hasTag("dbe:kaioken_x10")) {
          kaiokenSelection.kaioken_x10(player);
        } else if (player.hasTag("dbe:kaioken_x20")) {
          kaiokenSelection.kaioken_x20(player);
        } else
          system.runTimeout(() => {
            menu(player);
          }, 1);
        break;
      ////con
      //con x1
      case 1:
        if (
          getScore(player, "dbe:tp") >= ucPrice &&
          getScore(player, "dbe:statscon2") < 2500 &&
          getScore(player, "dbe:uc") == 1
        ) {
          player.runCommandAsync(
            "scoreboard players operation @s dbe:tp -= @s dbe:uc2"
          );
          player.runCommandAsync("scoreboard players add @s dbe:statscon2 1");
          //price increase x1
          player.runCommandAsync("scoreboard players add @s dbe:uc3 1");
          player.runCommandAsync("scoreboard players add @s dbe:uc2 1");
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc"
          );
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc3"
          );
          ////
          player.runCommandAsync("function stats");
          player.sendMessage({ rawtext: [{ text: "§a + 1 CON" }] });
        }
        //con x10
        else if (
          getScore(player, "dbe:tp") >= ucPrice &&
          getScore(player, "dbe:statscon2") < 2500 &&
          getScore(player, "dbe:uc") == 10
        ) {
          player.runCommandAsync(
            "scoreboard players operation @s dbe:tp -= @s dbe:uc2"
          );
          player.runCommandAsync("scoreboard players add @s dbe:statscon2 10");
          //price increase x10
          player.runCommandAsync("scoreboard players add @s dbe:uc3 1");
          player.runCommandAsync("scoreboard players add @s dbe:uc2 10");
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc"
          );
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc3"
          );
          ////
          player.runCommandAsync("function stats");
          player.sendMessage({ rawtext: [{ text: "§a + 10 CON" }] });
        }
        //con x100
        else if (
          getScore(player, "dbe:tp") >= ucPrice &&
          getScore(player, "dbe:statscon2") < 2500 &&
          getScore(player, "dbe:uc") == 100
        ) {
          player.runCommandAsync(
            "scoreboard players operation @s dbe:tp -= @s dbe:uc2"
          );
          player.runCommandAsync("scoreboard players add @s dbe:statscon2 100");
          //price increase x100
          player.runCommandAsync("scoreboard players add @s dbe:uc3 1");
          player.runCommandAsync("scoreboard players add @s dbe:uc2 100");
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc"
          );
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc3"
          );
          ////
          player.runCommandAsync("function stats");
          player.sendMessage({ rawtext: [{ text: "§a + 100 CON" }] });
        } else if (getScore(player, "dbe:statscon2") >= 2500) {
          player.sendMessage({ rawtext: [{ translate: "text.max" }] });
        } else
          player.sendMessage({ rawtext: [{ translate: "learn.error.1" }] });
        system.runTimeout(() => {
          if (getScore(player, "dbe:statscon2") > 2500) {
            player.runCommandAsync(
              "scoreboard players set @s dbe:statscon2 2500"
            );
            player.runCommandAsync("function stats");
          }
          attributes(player);
        }, 1);
        break;
      ////str
      //str x1
      case 2:
        if (
          getScore(player, "dbe:tp") >= ucPrice &&
          getScore(player, "dbe:statsstr2") < 2500 &&
          getScore(player, "dbe:uc") == 1
        ) {
          player.runCommandAsync(
            "scoreboard players operation @s dbe:tp -= @s dbe:uc2"
          );
          player.runCommandAsync("scoreboard players add @s dbe:statsstr2 1");
          //price increase x1
          player.runCommandAsync("scoreboard players add @s dbe:uc3 1");
          player.runCommandAsync("scoreboard players add @s dbe:uc2 1");
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc"
          );
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc3"
          );
          ////
          player.runCommandAsync("function stats");
          player.sendMessage({ rawtext: [{ text: "§a + 1 STR" }] });
        }
        //str x10
        else if (
          getScore(player, "dbe:tp") >= ucPrice &&
          getScore(player, "dbe:statsstr2") < 2500 &&
          getScore(player, "dbe:uc") == 10
        ) {
          player.runCommandAsync(
            "scoreboard players operation @s dbe:tp -= @s dbe:uc2"
          );
          player.runCommandAsync("scoreboard players add @s dbe:statsstr2 10");
          //price increase x10
          player.runCommandAsync("scoreboard players add @s dbe:uc3 1");
          player.runCommandAsync("scoreboard players add @s dbe:uc2 10");
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc"
          );
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc3"
          );
          ////
          player.runCommandAsync("function stats");
          player.sendMessage({ rawtext: [{ text: "§a + 10 STR" }] });
        }
        //str x100
        else if (
          getScore(player, "dbe:tp") >= ucPrice &&
          getScore(player, "dbe:statsstr2") < 2500 &&
          getScore(player, "dbe:uc") == 100
        ) {
          player.runCommandAsync(
            "scoreboard players operation @s dbe:tp -= @s dbe:uc2"
          );
          player.runCommandAsync("scoreboard players add @s dbe:statsstr2 100");
          //price increase x100
          player.runCommandAsync("scoreboard players add @s dbe:uc3 1");
          player.runCommandAsync("scoreboard players add @s dbe:uc2 100");
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc"
          );
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc3"
          );
          ////
          player.runCommandAsync("function stats");
          player.sendMessage({ rawtext: [{ text: "§a + 100 STR" }] });
        } else if (getScore(player, "dbe:statscon2") >= 2500) {
          player.sendMessage({ rawtext: [{ translate: "text.max" }] });
        } else
          player.sendMessage({ rawtext: [{ translate: "learn.error.1" }] });
        system.runTimeout(() => {
          if (getScore(player, "dbe:statsstr2") > 2500) {
            player.runCommandAsync(
              "scoreboard players set @s dbe:statsstr2 2500"
            );
            player.runCommandAsync("function stats");
          }
          attributes(player);
        }, 1);
        break;
      ////spe
      //spe x1
      case 3:
        if (
          getScore(player, "dbe:tp") >= ucPrice &&
          getScore(player, "dbe:statsspe") < 2500 &&
          getScore(player, "dbe:uc") == 1
        ) {
          player.runCommandAsync(
            "scoreboard players operation @s dbe:tp -= @s dbe:uc2"
          );
          player.runCommandAsync("scoreboard players add @s dbe:energy2 10");
          player.runCommandAsync("scoreboard players add @s dbe:energy3 10");
          player.runCommandAsync("scoreboard players add @s dbe:statsspe 1");
          //price increase x1
          player.runCommandAsync("scoreboard players add @s dbe:uc3 1");
          player.runCommandAsync("scoreboard players add @s dbe:uc2 1");
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc"
          );
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc3"
          );
          ////
          player.sendMessage({ rawtext: [{ text: "§a + 1 SPE" }] });
        }
        //spe x10
        else if (
          getScore(player, "dbe:tp") >= ucPrice &&
          getScore(player, "dbe:statsspe") < 2500 &&
          getScore(player, "dbe:uc") == 10
        ) {
          player.runCommandAsync(
            "scoreboard players operation @s dbe:tp -= @s dbe:uc2"
          );
          player.runCommandAsync("scoreboard players add @s dbe:energy2 100");
          player.runCommandAsync("scoreboard players add @s dbe:energy3 100");
          player.runCommandAsync("scoreboard players add @s dbe:statsspe 10");
          //price increase x10
          player.runCommandAsync("scoreboard players add @s dbe:uc3 1");
          player.runCommandAsync("scoreboard players add @s dbe:uc2 10");
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc"
          );
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc3"
          );
          ////
          player.sendMessage({ rawtext: [{ text: "§a + 10 SPE" }] });
        }
        //spe x100
        else if (
          getScore(player, "dbe:tp") >= ucPrice &&
          getScore(player, "dbe:statsspe") < 2500 &&
          getScore(player, "dbe:uc") == 100
        ) {
          player.runCommandAsync(
            "scoreboard players operation @s dbe:tp -= @s dbe:uc2"
          );
          player.runCommandAsync("scoreboard players add @s dbe:energy2 1000");
          player.runCommandAsync("scoreboard players add @s dbe:energy3 1000");
          player.runCommandAsync("scoreboard players add @s dbe:statsspe 100");
          //price increase x100
          player.runCommandAsync("scoreboard players add @s dbe:uc3 1");
          player.runCommandAsync("scoreboard players add @s dbe:uc2 100");
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc"
          );
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 += @s dbe:uc3"
          );
          ////
          player.sendMessage({ rawtext: [{ text: "§a + 100 SPE" }] });
        } else if (getScore(player, "dbe:statsspe") >= 2500) {
          player.sendMessage({ rawtext: [{ translate: "text.max" }] });
        } else
          player.sendMessage({ rawtext: [{ translate: "learn.error.1" }] });
        system.runTimeout(() => {
          if (getScore(player, "dbe:statsspe") > 2500) {
            player.runCommandAsync(
              "scoreboard players set @s dbe:statsspe 2500"
            );
          }
          attributes(player);
        }, 1);
        break;
      ////uc
      case 4:
        if (getScore(player, "dbe:uc") == 1) {
          player.runCommandAsync("scoreboard players set @s dbe:uc 10");
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 *= multiply10 dbe:operation"
          );
        } else if (getScore(player, "dbe:uc") == 10) {
          player.runCommandAsync("scoreboard players set @s dbe:uc 100");
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 *= multiply10 dbe:operation"
          );
        } else if (getScore(player, "dbe:uc") == 100) {
          player.runCommandAsync("scoreboard players set @s dbe:uc 1");
          player.runCommandAsync(
            "scoreboard players operation @s dbe:uc2 /= multiply100 dbe:operation"
          );
        }
        system.runTimeout(() => {
          attributes(player);
        }, 1);
        break;
    }
  });
}
//@menu
export function teleport(player) {
  const form = new ActionFormData();
  form.title("§3DBX Network GUI");
  form.button("ui.menu.button_2");
  form.button({ rawtext: [{ translate: "kaioken" }, { text: " (OFF)" }] });
  form.button("entity.dbe:shadow_dummy.name");
  form.button("ui.menu.button_5");
  form.button("ui.menu.button_6");
  form.show(player).then((response) => {
    switch (response.selection) {
      case 0:
        const overworldWarps = ["§bTournament", "§bLookout", "§bWest City", "§bEast City", "§bBuss House"];
        form = new MinecraftUi.ModalFormData()
        .title("§3§lOverworld Warps")
        .dropdown("\n\n§bSelect a place in the overworld to warp:\n\n§fGo somewhere in the server quickly!\n\n", overworldWarps);
        form.show(player).then(res => {
          if (res.canceled == true) return;
          successfulMessage(player, `§l§3Teleport §r§bSuccessfully warped to ${overworldWarps[res.formValues[0]]}.`)
          switch (res.formValues[0]) {
            case 0://tournament
              player.teleport({x:433, y:68, z:2383}, world.getDimension("overworld"), 0, 0)
              break;
            case 1://lookout
              player.teleport({x:331, y:246, z:3225}, world.getDimension("overworld"), 0, 0)
              break;
            case 2://west city
              player.teleport({x:1101, y:68, z:-901}, world.getDimension("overworld"), 0, 0)
              break;
            case 3://east city
              player.teleport({x:1364, y:69, z:3202}, world.getDimension("overworld"), 0, 0)
              break;
              case 4://buu
              player.teleport({x:1685, y:76, z:3255}, world.getDimension("overworld"), 0, 0)
              break;
          }
        });
        break;
        case 1:
          const planetWarps = ["§bTournament of Power", "§bHell", "§bBeerus Planet", "§bFreeza Ship"];
          form = new MinecraftUi.ModalFormData()
          .title("§3§lPlanet Warps")
          .dropdown("\n\n§bSelect a place in another planet to warp:\n\n§fGo somewhere in the server quickly!\n\n", planetWarps);
          form.show(player).then(res => {
            if (res.canceled == true) return;
            successfulMessage(player, `§l§3Teleport §r§bSuccessfully warped to ${planetWarps[res.formValues[0]]}.`)
            switch (res.formValues[0]) {
              case 0://top
                player.teleport({x:565, y:144, z:-986}, world.getDimension("the_end"), 0, 0)
                break;
              case 1://hell
                player.teleport({x:10056, y:112, z:10008}, world.getDimension("the_end"), 0, 0)
                break;
              case 2://beerus
                player.teleport({x:-9966, y:153, z:-9949}, world.getDimension("the_end"), 0, 0)
                break;
                 case 3://freeza
                player.teleport({x:-5973, y:107, z:5515}, world.getDimension("the_end"), 0, 0)
                break;
            }
          });
          break;
        case 2:
          const minigameWarps = ["§bDeath Star", "§bBoss Battles", "§bDungeons", "§bGTA", "§bDisney Land", "§bParkour", "§bMaze"];
          form = new MinecraftUi.ModalFormData()
          .title("§3§lMinigame Warps")
          .dropdown("\n\n§bSelect a place in minigames to warp:\n\n§fGo somewhere in the server quickly!\n\n", minigameWarps);
          form.show(player).then(res => {
            if (res.canceled == true) return;
            successfulMessage(player, `§l§3Teleport §r§bSuccessfully warped to ${minigameWarps[res.formValues[0]]}.`)
            switch (res.formValues[0]) {
              case 0://death star
                player.teleport({x:0, y:0, z:0}, world.getDimension("overworld"), 0, 0)
                break;
              case 1://boss
                player.teleport({x:4949, y:8, z:7809}, world.getDimension("overworld"), 0, 0)
                break;
              case 2://dungeon
                player.teleport({x:5093, y:6, z:7870}, world.getDimension("overworld"), 0, 0)
                break;
              case 3://gta
                player.teleport({x:10395, y:39, z:10205}, world.getDimension("overworld"), 0, 0)
                break;
              case 4://disney
                player.teleport({x:-9875, y:19, z:-9681}, world.getDimension("overworld"), 0, 0)
                break;
              case 5://parkour
                player.teleport({x:176, y:64, z:997}, world.getDimension("overworld"), 0, 0)
                break;
              case 6://maze
                player.teleport({x:-29614, y:7, z:-49649}, world.getDimension("overworld"), 0, 0)
                break;
              }
            });
          }
        })
      }